(menu-bar-mode -1)
(tool-bar-mode -1)
(scroll-bar-mode -1)
(tooltip-mode -1)
(fset 'yes-or-no-p 'y-or-n-p)
(fringe-mode 10)
(setq inhibit-startup-message t)
(setq initial-scratch-message "")
(setq locale-coding-system 'utf-8)
(set-terminal-coding-system 'utf-8)
(set-keyboard-coding-system 'utf-8)
(set-selection-coding-system 'utf-8)
(prefer-coding-system 'utf-8)
(setq inhibit-splash-screen t)
(setq visible-bell t)
(require 'iso-transl)
(setq dired-dwim-target t)
(icomplete-mode 1)
(eldoc-mode 1)

(require 'package)
(add-to-list
 'package-archives
 '("melpa" . "https://melpa.org/packages/")
 t)
(package-initialize)

(defun moritz/load-extra-fn-translations ()
  "Translate extra function keys (F13-20) to allow it to be used in terminal "
               ;;; TTY Key hacks
               ;;; The translations with the settings below fit the adjustments to the key table.
  (define-key input-decode-map "\e[25~" [(f13)])
  (define-key input-decode-map "\e[26~" [(f14)])
  (define-key input-decode-map "\e[28~" [(f15)])
  (define-key input-decode-map "\e[29~" [(f16)])
  (define-key input-decode-map "\e[31~" [(f17)])
  (define-key input-decode-map "\e[32~" [(f18)])
  (define-key input-decode-map "\e[33~" [(f19)])
  (define-key input-decode-map "\e[34~" [(f20)])
  (define-key input-decode-map "\e[35~" [(f21)])
  (define-key input-decode-map "\e[36~" [(f22)])
  (define-key input-decode-map "\e[37~" [(f23)])
  (define-key input-decode-map "\e[38~" [(f24)])
  (define-key input-decode-map "\e[39~" [(f25)])
  (define-key input-decode-map "\e[40~" [(f26)])
  (define-key input-decode-map "\e[41~" [(f27)])
  (define-key input-decode-map "\e[42~" [(f28)])
  (define-key input-decode-map "\e[43~" [(f29)])
  (define-key input-decode-map "\e[44~" [(f30)])
  (define-key input-decode-map "\e[45~" [(f31)])
  (define-key input-decode-map "\e[46~" [(f32)])
  (define-key input-decode-map "\e[47~" [(f33)])
  (define-key input-decode-map "\e[48~" [(f34)])
  (define-key input-decode-map "\e[49~" [(f35)])
  (define-key input-decode-map "\e[50~" [(f36)])
  (define-key input-decode-map "\e[51~" [(f37)])
  (define-key input-decode-map "\e[52~" [(f38)])
  (define-key input-decode-map "\e[53~" [(f39)])
  (define-key input-decode-map "\e[54~" [(f40)]))

(defun moritz/repeat-recent-compilation ()
  (interactive)
  (let ((comm (assoc 'compile command-history)))
    (if comm
        (eval comm)
      (compile "make -k"))))

(defun moritz/rebind-caps-lock-as-f13 ()
  (interactive)
  (shell-command "xmodmap -e 'clear Lock' -e 'keycode 66 = F13 Caps_Lock NoSymbol NoSymbol'"))

(defun moritz/goto-file-at-point (arg)
  (interactive "P")
  (if arg
      (find-file-other-window (thing-at-point 'filename))
    (find-file-at-point (thing-at-point 'filename))))

(if (eq window-system 'x)
    (moritz/rebind-caps-lock-as-f13)
  (progn
    (moritz/rebind-caps-lock-as-f13)
    (moritz/load-extra-fn-translations)))

(defun moritz/simple-replace-with-kill-ring ()
  (interactive)
  (save-excursion
    (if (not (use-region-p))
        (mosexp))
    (funcall-interactively 'delete-backward-char 1)
    (yank)))

(defun moritz/semicolon-dwin (arg)
  "Try to wonder to if the expected behaviour is to add a semicolon or
          comment out line depending in the context"
  (interactive "p")
  (let ((inserted nil))
    (save-excursion
      (if (and (derived-mode-p 'prog-mode)
               (= 41 (following-char))
               (progn
                 (forward-char 1)
                 (= 10 (following-char))))
          (progn
            (insert ";")
            (setq inserted t))))
    (if inserted
        (end-of-line)
      (comment-line arg))))

(defun moritz/kill-this-buffer ()
  (interactive)
  (kill-buffer (current-buffer)))

(global-set-key (kbd "C-x C-k") #'moritz/kill-this-buffer)
(global-set-key (kbd "C-x k") #'moritz/kill-this-buffer)

(load-theme 'wombat)

(define-key global-map (kbd "<f13>") 'emacs-nav-mode)

(defun moritz/set-font-size (&optional font-size)
  (interactive)
  (let ((font-size
         (if (numberp font-size)
             font-size
           (read-number "Type the font-height: " 160))))
    (set-face-attribute 'default nil
                        :height font-size
                        :weight 'normal
                        :width 'normal
                        )))
(moritz/set-font-size 160)

(unintern 'emacs-nav-minor-mode-map)
(unintern 'emacs-nav-mode-map)

(defun mosexp ()
  (interactive)
  (let ((naive-expand (mosexp--get-bounds 0))
	(conservative-expand (mosexp--get-bounds -1)))
    (message (format "%S" conservative-expand))
    (if (= (cadr naive-expand) (cadr conservative-expand))
	(mosexp--set-region (car conservative-expand)
			    (cadr conservative-expand))
      (mosexp--set-region (car naive-expand)
			  (cadr naive-expand)))))

(defun mosexp--set-region (beg end)
  (set-mark beg)
  (goto-char end))

(defun mosexp--get-bounds (sexps-to-forward)
  (save-excursion
    (condition-case nil
	(forward-sexp sexps-to-forward)
      (error nil))
    (let ((beg nil)
	  (end nil))
      (condition-case nil
	  (mark-sexp 1 nil)
	(error nil))
      (if (use-region-p)
	  (progn
	    (setq beg (region-beginning))
	    (setq end (region-end)))
	(error "Failed to get sexp bounds"))
      (deactivate-mark)
      `(,beg ,end))))

(defun moritz/save-buffer (arg)
  (interactive "p")
  (let ((arg (if (not arg)
		 1
	       arg)))
    (if (bound-and-true-p org-src-mode)
	(org-edit-src-save)
      (save-buffer))))


(defun moritz/next (arg)
  (interactive "p")
  (let ((arg (if (not arg)
		 1
	       arg)))
    (cond
     ((and (eq major-mode 'org-mode)
	   (= (following-char) 42)
	   (= (current-column) 0))
      (org-next-visible-heading arg))
     (t (next-line arg)))))

(defun moritz/previous (arg)
  (interactive "p")
  (let ((arg (if (not arg)
		 1
	       arg)))
    (cond
     ((and (eq major-mode 'org-mode)
	   (= (following-char) 42)
	   (= (current-column) 0))
      (org-previous-visible-heading arg))
     (t (next-line (* -1 arg))))))

(defun moritz/org-self-insert-command (arg)
       "Use org speed commands for previous visible heading"
       (interactive "p")
       (let ((arg (if (not arg)
                      1
                    arg)))
         (if (and (eq major-mode 'org-mode)
                  (= (following-char) 42)
                  (= (current-column) 0))
             (org-self-insert-command arg)
           (undo arg))))

(defun emacs-nav--disable-advice (orig-fun &rest args)
  (let ((emacs-nav-active (if emacs-nav-mode
                              t
                            nil)))
    (if emacs-nav-active
        (emacs-nav-mode -1))
    (apply orig-fun command args)))

(defun emacs-nav--disable-enable-advice (orig-fun &rest args)
  (message "Function called with emacs nav minor mode active")
  (let ((emacs-nav-active (if emacs-nav-mode
                              t
                            nil)))
    (if emacs-nav-active
        (emacs-nav-mode -1))
    (setq returning-value (apply orig-fun args))
    (if emacs-nav-active
        (emacs-nav-mode))
    returning-value))

(defun emacs-nav--disable-advice (orig-fun &rest args)
  (message "Function called with emacs nav minor mode active")
  (emacs-nav-mode -1)
  (apply orig-fun args))

(advice-add 'read-number :around #'emacs-nav--disable-enable-advice)
(advice-add 'read-string :around #'emacs-nav--disable-enable-advice)
;; (advice-add 'read-extended-command :around #'emacs-nav--disable-enable-advice)
;;(advice-add 'find-file-read-args :around #'emacs-nav--disable-enable-advice)
(advice-add 'read-from-minibuffer :around #'emacs-nav--disable-enable-advice)


;; (advice-add 'mc/mark-all-in-region-regexp :around #'emacs-nav--disable-enable-advice)

(require 'thingatpt)

(defun thing-at-point--end-of-integer ()
  "Go to the end of the integer at point."
  (let ((inhibit-changing-match-data t))
    ;; Skip over optional sign
    (when (looking-at "[+-]")
      (forward-char 1))
    ;; Skip over digits
    (skip-chars-forward "[:digit:]")
    ;; Check for at least one digit
    (unless (looking-back "[[:digit:]]")
      (error "No integer here"))))

(defun thing-at-point--beginning-of-integer ()
  "Go to the beginning of the integer at point."
  (let ((inhibit-changing-match-data t))
    ;; Skip backward over digits
    (skip-chars-backward "[:digit:]")
    ;; Check for digits and optional sign
    (unless (looking-at "[+-]?[[:digit:]]")
      (error "No integer here"))
    ;; Skip backward over optional sign
    (when (looking-back "[+-]")
      (backward-char 1))))

(defun thing-at-point--bounds-of-integer-at-point ()
  "Get boundaries of integer at point."
  (save-excursion
    (let (beg end)
      (thing-at-point--beginning-of-integer)
      (setq beg (point))
      (thing-at-point--end-of-integer)
      (setq end (point))
      (cons beg end))))

(defun thing-at-point-integer-at-point ()
  "Get integer at point."
  (let ((bounds (bounds-of-thing-at-point 'integer)))
    (string-to-number (buffer-substring (car bounds) (cdr bounds)))))

(put 'integer 'beginning-op 'thing-at-point--beginning-of-integer)
(put 'integer 'end-op 'thing-at-point--end-of-integer)
(put 'integer 'bounds-of-thing-at-point 'thing-at-point--bounds-of-integer-at-point)
(put 'integer 'thing-at-point 'thing-at-point-integer-at-point)


(defun moritz/operate-number-at-point (operation &optional arg)
  (interactive "p")
  (if (not (numberp (thing-at-point 'integer)))
      (error "thing at point is not a number"))
  (let ((start-pos (point))
	(operand (or arg 1))
	(value (thing-at-point 'integer)))
    (save-excursion
      (let ((bounds (bounds-of-thing-at-point 'integer)))
	(kill-region (car bounds) (cdr bounds))
	(insert (number-to-string (funcall operation value operand)))))
    (goto-char start-pos)))

(defun moritz/multiply-number-at-point (&optional arg)
  (interactive "p")
  (moritz/operate-number-at-point '* arg))

(defun moritz/divide-number-at-point (&optional arg)
  (interactive "p")
  (moritz/operate-number-at-point '/ arg))

(defun moritz/increment-number-at-point (&optional arg)
  (interactive "p")
  (moritz/operate-number-at-point '+ arg))

(defun moritz/decrement-number-at-point (&optional arg)
  (interactive "p")
  (let ((increment (or arg 1)))
    (moritz/increment-number-at-point (- 0 increment))))

(defun moritz/add-key-event (key)
  (setq unread-command-events (listify-key-sequence (kbd (format "%s" key)))))

(defun moritz/add-key-digit-M-0 ()
  (interactive)
  (moritz/add-key-event "M-0"))

(defun moritz/add-key-digit-M-1 ()
  (interactive)
  (moritz/add-key-event "M-1"))

(defun moritz/add-key-digit-M-2 ()
  (interactive)
  (moritz/add-key-event "M-2"))

(defun moritz/add-key-digit-M-3 ()
  (interactive)
  (moritz/add-key-event "M-3"))

(defun moritz/add-key-digit-M-4 ()
  (interactive)
  (moritz/add-key-event "M-4"))

(defun moritz/add-key-digit-M-5 ()
  (interactive)
  (moritz/add-key-event "M-5"))

(defun moritz/add-key-digit-M-6 ()
  (interactive)
  (moritz/add-key-event "M-6"))

(defun moritz/add-key-digit-M-7 ()
  (interactive)
  (moritz/add-key-event "M-7"))

(defun moritz/add-key-digit-M-8 ()
  (interactive)
  (moritz/add-key-event "M-8"))

(defun moritz/add-key-digit-M-9 ()
  (interactive)
  (moritz/add-key-event "M-9"))

(defun moritz/add-key-digit-Cc-Cc ()
  (interactive)
  (moritz/add-key-event "C-c C-c"))

(defun moritz/add-key-digit-Cc-Ck ()
  (interactive)
  (moritz/add-key-event "C-c C-k"))

(defun moritz/add-key-digit-Cc-Cz ()
  (interactive)
  (moritz/add-key-event "C-c C-z"))

(defun moritz/add-key-digit-Cc-Ct ()
  (interactive)
  (moritz/add-key-event "C-c C-t"))

(defun moritz/add-key-digit-Cc-Cv-Ct ()
  (interactive)
  (moritz/add-key-event "C-c C-v C-t"))

(defun moritz/add-key-digit-Cc-Cx-Cv ()
  (interactive)
  (moritz/add-key-event "C-c C-x C-v"))

(defun moritz/add-key-digit-Cc-Co ()
  (interactive)
  (moritz/add-key-event "C-c C-o"))

(defun moritz/add-key-digit-Cc-a ()
  (interactive)
  (moritz/add-key-event "C-c a"))

(defun moritz/add-key-digit-Cc-Cs ()
  (interactive)
  (moritz/add-key-event "C-c C-s"))

(defun moritz/add-key-digit-Cc-Cx-Ci ()
  (interactive)
  (moritz/add-key-event "C-c C-x C-i"))

(defun moritz/add-key-digit-Cc-Cx-Co ()
  (interactive)
  (moritz/add-key-event "C-c C-x C-o"))


(defun moritz/nav-mappings (map)
  (define-key map (kbd "n") 'moritz/next)
  (define-key map (kbd "p") 'moritz/previous)
  (define-key map (kbd "b") 'backward-char)
  (define-key map (kbd "f") 'forward-char)
  (define-key map (kbd "N") 'scroll-up-command)
  (define-key map (kbd "P") 'scroll-down-command)
  (define-key map (kbd "a") 'back-to-indentation)
  (define-key map (kbd "e") 'move-end-of-line)
  (define-key map (kbd "l") 'recenter-top-bottom)
  (define-key map (kbd "o") 'other-window)
  (define-key map (kbd "O") '(lambda (n)
                               (interactive "P")
                               (let ((current-prefix-arg 4)
                                     (n (if (not n)
                                            -1
                                          (* -1 n))))
                                 (other-window n))))
  (define-key map (kbd "v") 'mosexp)
  (define-key map (kbd ",") 'beginning-of-buffer)
  (define-key map (kbd ".") 'end-of-buffer))

(defun moritz/digit-arg-mappings (map)
  (define-key map (kbd "0") 'moritz/add-key-digit-M-0)
  (define-key map (kbd "1") 'moritz/add-key-digit-M-1)
  (define-key map (kbd "2") 'moritz/add-key-digit-M-2)
  (define-key map (kbd "3") 'moritz/add-key-digit-M-3)
  (define-key map (kbd "4") 'moritz/add-key-digit-M-4)
  (define-key map (kbd "5") 'moritz/add-key-digit-M-5)
  (define-key map (kbd "6") 'moritz/add-key-digit-M-6)
  (define-key map (kbd "7") 'moritz/add-key-digit-M-7)
  (define-key map (kbd "8") 'moritz/add-key-digit-M-8)
  (define-key map (kbd "9") 'moritz/add-key-digit-M-9))

(defun moritz/marker-mappings (map)
  (define-key map (kbd ";") 'moritz/semicolon-dwin)
  (define-key map (kbd "SPC") 'set-mark-command))

(defun moritz/rectangle-keys (map)
  (define-key map (kbd "x SPC") 'rectangle-mark-mode)
  (define-key map (kbd "x y") 'yank-rectangle)
  (define-key map (kbd "x w") 'kill-rectangle)
  (define-key map (kbd "x ç") 'copy-rectangle-as-kill))

(defun moritz/compilation-mappings (map)
  (define-key map (kbd "c m") 'moritz/repeat-recent-compilation))

(defun moritz/system-keys (map)
  (define-key map (kbd "M-x") 'execute-extended-command)
  (define-key map (kbd "x x") 'execute-extended-command)
  (define-key map (kbd "x h") 'mark-whole-buffer)
  (define-key map (kbd "x s") 'moritz/save-buffer)
  (define-key map (kbd "x b") 'switch-to-buffer)
  (define-key map (kbd "x f") 'find-file)
  (define-key map (kbd "x p") 'next-buffer)
  (define-key map (kbd "x n") 'previous-buffer)
  (define-key map (kbd "x k") 'moritz/kill-this-buffer)
  (define-key map (kbd "x d") 'dired)
  (define-key map (kbd "x z") 'repeat)
  (define-key map (kbd "x 0") 'delete-window)
  (define-key map (kbd "x 1") 'delete-other-windows)
  (define-key map (kbd "x 2") 'split-window-below)
  (define-key map (kbd "x 3") 'split-window-right)
  (define-key map (kbd "x e") 'eval-last-sexp)
  (define-key map (kbd "x r") 'eval-and-replace)
  (define-key map (kbd "u") 'moritz/org-self-insert-command)
  (define-key map (kbd "/") 'undo)
  (define-key map (kbd "x z") 'repeat)
  (define-key map (kbd "d") 'delete-char))

(defun moritz/kill-ring-save-symbol-at-point()
  (interactive)
  (require 'thingatpt)
  (if (use-region-p)
      (kill-ring-save (region-beginning) (region-end))
    (kill-new (thing-at-point 'symbol))))

(defun moritz/killer-mappings (map)
  (define-key map (kbd "k k") 'kill-line)
  (define-key map (kbd "k l") 'kill-whole-line)
  (define-key map (kbd "k r") '(lambda ()
                                 (interactive)
                                 (kill-region (region-beginning)
                                              (region-end))))
  (define-key map (kbd "ç") 'moritz/kill-ring-save-symbol-at-point)
  (define-key map (kbd "w") (lambda ()
                              (interactive)
                              (kill-region (region-beginning)
                                           (region-end))
                              (emacs-nav-mode -1))))

(defun moritz/goto-mappings ()
  (define-key map (kbd "g c") '(lambda ()
                                 (interactive)
                                 (org-clock-jump-to-current-clock)))
  (define-key map (kbd "g w") 'window-configuration-to-register)
  (define-key map (kbd "g SPC") 'point-to-register)
  (define-key map (kbd "g j") 'jump-to-register)
  (define-key map (kbd "g f") 'moritz/goto-file-at-point)
  (define-key map (kbd "g g") 'goto-line))

(defun moritz/help (map)
  (define-key map (kbd "h f") 'describe-function)
  (define-key map (kbd "h k") 'describe-key)
  (define-key map (kbd "h v") 'describe-variable))

(defun moritz/yanker-mappings (map)
  (define-key map (kbd "y") 'yank))

(define-key isearch-mode-map (kbd "P") 'isearch-repeat-backward)
(define-key isearch-mode-map (kbd "N") 'isearch-repeat-forward)

(defun moritz/search-replace-mappings (map)
  (define-key map (kbd "s") 'isearch-forward)
  (define-key map (kbd "j") 'isearch-forward)
  (define-key map (kbd "R") 'query-replace)
  (define-key map (kbd "r") 'moritz/simple-replace-with-kill-ring))

(defun moritz/ansi-term ()
  (interactive)
  (ansi-term "/usr/bin/zsh"))

(define-prefix-command 'moritz/terminal-map)
(define-key 'moritz/terminal-map (kbd ">") 'moritz/ansi-term)
(define-key 'moritz/terminal-map (kbd "g") 'moritz/gnome-terminal)

(defun moritz/terminal-mappings (map)
  (define-key map (kbd ">") 'moritz/terminal-map)
  (define-key map (kbd "c e") 'eshell))

(defun moritz/org-mappings (map)
  (define-key map (kbd "c x o") 'moritz/add-key-digit-Cc-Cx-Co)
  (define-key map (kbd "c x i") 'moritz/add-key-digit-Cc-Cx-Ci)
  (define-key map (kbd "c x l") 'org-clock-in-last)
  (define-key map (kbd "c c") 'moritz/add-key-digit-Cc-Cc)
  (define-key map (kbd "c k") 'moritz/add-key-digit-Cc-Ck)
  (define-key map (kbd "c z") 'moritz/add-key-digit-Cc-Cz)
  (define-key map (kbd "c t") 'moritz/add-key-digit-Cc-Ct)
  (define-key map (kbd "c v t") 'moritz/add-key-digit-Cc-Cv-Ct)
  (define-key map (kbd "c x v") 'moritz/add-key-digit-Cc-Cx-Cv)
  (define-key map (kbd "c o") 'moritz/add-key-digit-Cc-Co)
  (define-key map (kbd "c a") 'moritz/add-key-digit-Cc-a)
  (define-key map (kbd "c s") 'moritz/add-key-digit-Cc-Cs)
  (define-key map (kbd "z") 'moritz/org-edit))

(defun moritz/number-manipulation-mappings (map)
  (define-key map (kbd "<kp-add>") 'moritz/increment-number-at-point)
  (define-key map (kbd "<kp-subtract>") 'moritz/decrement-number-at-point)
  (define-key map (kbd "<kp-multiply>") 'moritz/multiply-number-at-point)
  (define-key map (kbd "<kp-divide>") 'moritz/divide-number-at-point))

(defvar emacs-nav-minor-mode-map
  (let ((map (make-sparse-keymap)))
    (moritz/nav-mappings map)
    (moritz/digit-arg-mappings map)
    (moritz/yanker-mappings map)
    (moritz/marker-mappings map)
    (moritz/system-keys map)
    (moritz/rectangle-keys map)
    (moritz/help map)
    (moritz/number-manipulation-mappings map)
    (moritz/search-replace-mappings map)
    (moritz/org-mappings map)
    (moritz/terminal-mappings map)
    (moritz/goto-mappings)
    (moritz/killer-mappings map)
    (moritz/compilation-mappings map)
    (define-key map (kbd "<f13>") 'emacs-nav-mode)
    map)
  "The keymap used when `emacs-nav-minor-mode' is active.")

(defun moritz/change-terminal-cursor (cursor)
  (shell-command (concat "dconf write /org/gnome/terminal/legacy/profiles:/:b1dcc9dd-5262-4d8d-a863-c897e6d979b9/cursor-shape \"'" cursor "'\"")))

(define-minor-mode emacs-nav-mode
  "Emacs navigation mode"
  :lighter " nav"
  :keymap emacs-nav-minor-mode-map
  :global 1

  (if emacs-nav-mode
      (progn
        (if (not (boundp 'old-cursor-color))
            (setq old-cursor-color (cdr (assoc 'cursor-color (frame-parameters)))))
        (set-cursor-color "magenta")
        (if (not (eq window-system 'x))
            (moritz/change-terminal-cursor "block")))
    (progn
      (set-cursor-color old-cursor-color)
      (if (not (eq window-system 'x))
          (moritz/change-terminal-cursor "ibeam")))))

(setenv "HOME" (concat "/home/" (getenv "USER")))

(setq
 org-todo-keywords
 '((sequence "TODO(t)" "SCHEDULED(s)" "WORKING(w)" "FOLLOWUP(f)" "VERIFY(v)" "HOLD(h)" "|" "DONE(d)" "CANCELED(c)" "SKIP(S)" "LOG(l)" "DELEGATED(D)")))

(setq org-todo-keyword-faces
      '(("TODO" . "wheat")
	("TODO" . "DarkGoldenrod")
	("WORKING" . "blue")
	("FOLLOWUP" . "SkyBlue")
	("VERIFY" . "IndianRed")
	("HOLD" . "dark khaki")
	("DONE" . (:foreground "lightgreen" :weight bold))
	("CANCELED" . "red")
	("SKIP" . "dim gray")
	("LOG" . "lightgreen")
	("DELEGATED" . "SpringGreen")))
